<?php
// $Id:

/**
 * This optional script is a more advanced version of the missing_files_listener.php. It uses the missing files report in conjunction 
 * with airing data on on the Drupal side to transfer the first 5 files to be aired within a given range. This is an attempt to prevent a
 * fill up of the media disc that could be caused by transfering the entire missing files report. Optionally, it will notify station
 * personel if a file has failed to transfer, completed transfers, or successfully run. This file may be cronned or called at will.
 * Note the Drupal Bootstrap.
 */

// This part of the script will be server specific. Script WILL NOT FUNCTION without the information below:

// Bootstrap Drupal:

chdir($defaultfilepath);
require_once './includes/bootstrap.inc';
drupal_bootstrap(DRUPAL_BOOTSTRAP_FULL);

// Station Specific Variables

// Some method of retrieving or hardcoding your channel number.
$channel = NULL;
// Events to email upon: failure, completion, completion_with_transfer. If you don't want mail comment out this line.
$notify_on = array('failure', 'completion_with_transfer');
// Emails to contact when files fail to transfer
$notification_contacts = array('email'); 
// Mount Path
$mount = NULL;
// Path to your missing files report mount.url:3000/missing_files.xml
$url = NULL; 
$xml = simplexml_load_file($url);
// Path to Drupal install from server root.
$defaultfilepath = NULL; 

// Script Mechanic Variables: Do whatever you want here.
// If you don't have OPS installed you'll have to set this some other way.
$channel_tid = om_playback_servers_get_tid_from_channel_number($channel);
// Number of days out to transfer files.
$days_out_to_transfer = 7;
// Number of shows ot transfer per execution.
$shows_to_transfer_per_run = 5;

$importpath = '/mnt/' . $mount . '/import/media/';
print 'Missing Files Report: ' . $url . '<br />';
print 'Default: ' . $defaultfilepath . '<br />';
print 'Import: ' . $importpath;

// End server specific changes

$unique_shows = array();

$size = 0;

/** 
 * Look up most near airings for each file name. Have to use dirty string comparison.
 * If you aren't concerned with caps comparison substitute LIKE BINARY with '=' When new
 * Princeton API changes are made, missing files report should contain air time rendering
 * this whole process obsolete.
 */
 
foreach ($xml->content_file AS $file) {
  $airing_query = db_query('
                    SELECT * 
                    FROM {content_type_om_airing}
                    JOIN {term_node} ON content_type_om_airing.nid = term_node.nid
                    WHERE field_om_airing_mpeg2_value LIKE BINARY "%s"
                    AND term_node.tid = %d
                    ORDER BY field_om_airing_date_value DESC
                    ', $file->filename, $channel_tid
                  );
                  
  while ($airing = db_fetch_array($airing_query)) {
    // if the files most recent airing is near enough to days out continue.
    if (strtotime($airing['field_om_airing_date_value'] .'UTC') < (time() + ($days_out_to_transfer * 24 * 60 * 60)) && strtotime($airing['field_om_airing_date_value'] . 'UTC') > strtotime(date('Y'))) {
      // if we haven't exceeded the total number of transfers per run add to transfer list.
      if (count($file_list) < $shows_to_transfer_per_run) {
        $file_list[] = $airing['field_om_airing_mpeg2_value'];
      }
      $unique_shows = array_unique($file_list);
    }
  }
}

print '<h3>Copying shows</h3>';
if (!$unique_shows) {
  print 'Missing files report is empty.';
}
else {
  print '<table><tr><th>file to transfer</th>path from:</th><th>path to:</th></tr>';
}

foreach ($unique_shows as $show) {
  
  $filepath = db_result(db_query('SELECT filepath FROM {files} JOIN {content_type_om_show} ON field_om_show_mpeg2_fid = files.fid WHERE filename = "%s"', $show));
  
  $result = copy($defaultfilepath.$filepath, $importpath.$show);
  
  if ($result) {
    // If Success Continue.
    $output .= ' ' . $show . '<br/>';
    print '<tr><td>' . $show . '</td>' . '<td>Failed: ' . $defaultfilepath.$filepath . '<td><td>Failed:' . $importpath.$show . '<td></tr>';
  }
  else {
    // If Fail: Check to see if we need to notify anyone.
    
    if (in_array('failure', $notify_on) && !empty($notification_contacts)) {
      foreach ($notification_contacts as $contact) {
        $module = 'om_playback_servers_princeton';
        $key = 'missing_files_listener';
        $language = language_default();
        $params = array('channel' => $channel, 'filepath' => $filepath, 'defaultfilepath' => $defaultfilepath);
        $from = 'Missing File Transfer Script';
        $send = TRUE;
        $message = drupal_mail($module, $key, $contact, $language, $params, $from, $send);
      }
    }
    
    print '<tr><td>' . $show . '</td>' . '<td>' . $defaultfilepath.$filepath . '<td><td>' . $importpath.$show . '<td></tr>';
  }
  
}
if ($unique_shows) {
  print '</table>';
}

if (!empty($notification_contacts) && in_array('completion', $notify_on)) {
  if ($output) {
    $output = 'Successfully transfered: <br/>' . $output;
  } else {
    $output = 'The transfer script ran, but there were no files to transfer.';
  }
  foreach ($notification_contacts as $contact) {
    $module = 'om_playback_servers_princeton';
    $key = 'missing_files_listener_run_notice';
    $language = language_default();
    $from = 'Missing File Transfer Script';
    $params = array('output' => $output);
    $send = TRUE;
    $message = drupal_mail($module, $key, $contact, $language, $params, $from, $send);
  }
}

if (!empty($notification_contacts) && in_array('completion_with_transfer', $notify_on) && $output) {
  foreach ($notification_contacts as $contact) {
    $output = 'Successfully transfered: <br/>' . $output;
    $module = 'om_playback_servers_princeton';
    $key = 'missing_files_listener_run_notice';
    $language = language_default();
    $from = 'Missing File Transfer Script';
    $params = array('output' => $output);
    $send = TRUE;
    $message = drupal_mail($module, $key, $contact, $language, $params, $from, $send);
  }
}

?>
